# Generated by ariadne-codegen

from __future__ import annotations

from typing import Any, Literal, TypeVar

from pydantic import BaseModel, ConfigDict, Field, Json, ValidationError, WrapValidator
from pydantic.alias_generators import to_camel
from pydantic.main import IncEx
from pydantic_core import to_json
from pydantic_core.core_schema import ValidatorFunctionWrapHandler

from .typing_compat import Annotated, override


# Base class for all automation classes/types.
# Omitted from docstring to avoid inclusion in generated docs.
class Base(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        validate_default=True,
        extra="forbid",
        alias_generator=to_camel,
        use_attribute_docstrings=True,
        from_attributes=True,
        revalidate_instances="always",
    )

    @override
    def model_dump(
        self,
        *,
        mode: Literal["json", "python"] | str = "json",  # NOTE: changed default
        include: IncEx | None = None,
        exclude: IncEx | None = None,
        context: dict[str, Any] | None = None,
        by_alias: bool = True,  # NOTE: changed default
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
        round_trip: bool = True,  # NOTE: changed default
        warnings: bool | Literal["none", "warn", "error"] = True,
        serialize_as_any: bool = False,
    ) -> dict[str, Any]:
        return super().model_dump(
            mode=mode,
            include=include,
            exclude=exclude,
            context=context,
            by_alias=by_alias,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
            round_trip=round_trip,
            warnings=warnings,
            serialize_as_any=serialize_as_any,
        )

    @override
    def model_dump_json(
        self,
        *,
        indent: int | None = None,
        include: IncEx | None = None,
        exclude: IncEx | None = None,
        context: dict[str, Any] | None = None,
        by_alias: bool = True,  # NOTE: changed default
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
        round_trip: bool = True,  # NOTE: changed default
        warnings: bool | Literal["none", "warn", "error"] = True,
        serialize_as_any: bool = False,
    ) -> str:
        return super().model_dump_json(
            indent=indent,
            include=include,
            exclude=exclude,
            context=context,
            by_alias=by_alias,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
            round_trip=round_trip,
            warnings=warnings,
            serialize_as_any=serialize_as_any,
        )


# Base class with extra customization for GQL generated types.
# Omitted from docstring to avoid inclusion in generated docs.
class GQLBase(Base):
    model_config = ConfigDict(
        extra="ignore",
        protected_namespaces=(),
    )


# ------------------------------------------------------------------------------
# Reusable annotations for field types
T = TypeVar("T")

GQLId = Annotated[
    str,
    Field(repr=False, strict=True, frozen=True),
]

Typename = Annotated[
    T,
    Field(repr=False, alias="__typename", frozen=True),
]


def validate_maybe_json(v: Any, handler: ValidatorFunctionWrapHandler) -> Any:
    """Wraps default Json[...] field validator to allow instantiation with an already-decoded value."""
    try:
        return handler(v)
    except ValidationError:
        # Try revalidating after properly jsonifying the value
        return handler(to_json(v, by_alias=True, round_trip=True))


SerializedToJson = Annotated[
    Json[T],
    # Allow lenient instantiation/validation: incoming data may already be deserialized.
    WrapValidator(validate_maybe_json),
]



# Generated by ariadne-codegen

"""Definitions to ensure compatibility with all supported python versions."""

import sys

if sys.version_info >= (3, 12):
    from typing import Annotated, override
else:
    from typing_extensions import Annotated, override


Annnotated = Annotated
override = override



"""A message protocol for the internal service process.

The core of W&B is implemented by a side process that asynchronously uploads
data. The client process (such as this Python code) sends requests to the
service, and for some requests, the service eventually sends a response.

The client can send multiple requests before the service provides a response.
The Mailbox handles matching responses to requests. An internal thread
continuously reads data from the service and passes it to the mailbox.
"""

from .handles import HandleAbandonedError, MailboxHandle
from .mailbox import Mailbox, MailboxClosedError
from .wait_with_progress import wait_all_with_progress, wait_with_progress

__all__ = [
    "HandleAbandonedError",
    "MailboxHandle",
    "Mailbox",
    "MailboxClosedError",
    "wait_all_with_progress",
    "wait_with_progress",
]



# Generated by ariadne-codegen
# Source: tools/graphql_codegen/utils/

__all__ = ["SERVER_FEATURES_QUERY_GQL"]

SERVER_FEATURES_QUERY_GQL = """
query ServerFeaturesQuery {
  serverInfo {
    features {
      name
      isEnabled
    }
  }
}
"""



from __future__ import annotations

import asyncio
import math
import threading
from typing import TYPE_CHECKING

from wandb.proto import wandb_internal_pb2 as pb

# Necessary to break an import loop.
if TYPE_CHECKING:
    from wandb.sdk.interface import interface


class HandleAbandonedError(Exception):
    """The handle has no result and has been abandoned."""


class MailboxHandle:
    """A thread-safe handle that allows waiting for a response to a request."""

    def __init__(self, address: str) -> None:
        self._address = address
        self._lock = threading.Lock()
        self._event = threading.Event()

        self._abandoned = False
        self._result: pb.Result | None = None

        self._asyncio_events: dict[asyncio.Event, _AsyncioEvent] = dict()

    def deliver(self, result: pb.Result) -> None:
        """Deliver the response.

        This may only be called once. It is an error to respond to the same
        request more than once. It is a no-op if the handle has been abandoned.
        """
        with self._lock:
            if self._abandoned:
                return

            if self._result:
                raise ValueError(
                    f"A response has already been delivered to {self._address}."
                )

            self._result = result
            self._signal_done()

    def cancel(self, iface: interface.InterfaceBase) -> None:
        """Cancel the handle, requesting any associated work to not complete.

        This automatically abandons the handle, as a response is no longer
        guaranteed.

        Args:
            interface: The interface on which to publish the cancel request.
        """
        iface.publish_cancel(self._address)
        self.abandon()

    def abandon(self) -> None:
        """Abandon the handle, indicating it will not receive a response."""
        with self._lock:
            self._abandoned = True
            self._signal_done()

    def _signal_done(self) -> None:
        """Indicate that the handle either got a result or became abandoned.

        The lock must be held.
        """
        # Unblock threads blocked on `wait_or`.
        self._event.set()

        # Unblock asyncio loops blocked on `wait_async`.
        for asyncio_event in self._asyncio_events.values():
            asyncio_event.set_threadsafe()
        self._asyncio_events.clear()

    def check(self) -> pb.Result | None:
        """Returns the result if it's ready."""
        with self._lock:
            return self._result

    def wait_or(self, *, timeout: float | None) -> pb.Result:
        """Wait for a response or a timeout.

        This is called `wait_or` because it replaces a method called `wait`
        with different semantics.

        Args:
            timeout: A finite number of seconds or None to never time out.
                If less than or equal to zero, times out immediately unless
                the result is available.

        Returns:
            The result if it arrives before the timeout or has already arrived.

        Raises:
            TimeoutError: If the timeout is reached.
            HandleAbandonedError: If the handle becomes abandoned.
        """
        if timeout is not None and not math.isfinite(timeout):
            raise ValueError("Timeout must be finite or None.")

        if not self._event.wait(timeout=timeout):
            raise TimeoutError(
                f"Timed out waiting for response on {self._address}",
            )

        with self._lock:
            if self._result:
                return self._result

            assert self._abandoned
            raise HandleAbandonedError()

    async def wait_async(self, *, timeout: float | None) -> pb.Result:
        """Wait for a response or timeout.

        This must run in an `asyncio` event loop.

        Args:
            timeout: A finite number of seconds or None to never time out.

        Returns:
            The result if it arrives before the timeout or has already arrived.

        Raises:
            TimeoutError: If the timeout is reached.
            HandleAbandonedError: If the handle becomes abandoned.
        """
        if timeout is not None and not math.isfinite(timeout):
            raise ValueError("Timeout must be finite or None.")

        evt = asyncio.Event()
        self._add_asyncio_event(asyncio.get_event_loop(), evt)

        try:
            await asyncio.wait_for(evt.wait(), timeout=timeout)

        except (asyncio.TimeoutError, TimeoutError) as e:
            with self._lock:
                if self._result:
                    return self._result
                elif self._abandoned:
                    raise HandleAbandonedError()
                else:
                    raise TimeoutError(
                        f"Timed out waiting for response on {self._address}"
                    ) from e

        else:
            with self._lock:
                if self._result:
                    return self._result

                assert self._abandoned
                raise HandleAbandonedError()

        finally:
            self._forget_asyncio_event(evt)

    def _add_asyncio_event(
        self,
        loop: asyncio.AbstractEventLoop,
        event: asyncio.Event,
    ) -> None:
        """Add an event to signal when a result is received.

        If a result already exists, this notifies the event loop immediately.
        """
        asyncio_event = _AsyncioEvent(loop, event)

        with self._lock:
            if self._result or self._abandoned:
                asyncio_event.set_threadsafe()
            else:
                self._asyncio_events[event] = asyncio_event

    def _forget_asyncio_event(self, event: asyncio.Event) -> None:
        """Cancel signalling an event when a result is received."""
        with self._lock:
            self._asyncio_events.pop(event, None)


class _AsyncioEvent:
    def __init__(
        self,
        loop: asyncio.AbstractEventLoop,
        event: asyncio.Event,
    ):
        self._loop = loop
        self._event = event

    def set_threadsafe(self) -> None:
        """Set the asyncio event in its own loop."""
        self._loop.call_soon_threadsafe(self._event.set)



from wandb.sdk.internal.internal_api import Api as InternalApi

ARTIFACTS_TYPES_FRAGMENT = """
fragment ArtifactTypesFragment on ArtifactTypeConnection {
    edges {
         node {
             id
             name
             description
             createdAt
         }
         cursor
    }
    pageInfo {
        endCursor
        hasNextPage
    }
}
"""

ARTIFACT_FILES_FRAGMENT = """fragment ArtifactFilesFragment on Artifact {
    files(names: $fileNames, after: $fileCursor, first: $fileLimit) {
        edges {
            node {
                id
                name: displayName
                url
                sizeBytes
                storagePath
                mimetype
                updatedAt
                digest
                md5
                directUrl
            }
            cursor
        }
        pageInfo {
            endCursor
            hasNextPage
        }
    }
}"""


def _gql_artifact_fragment(include_aliases: bool = True) -> str:
    """Return a GraphQL query fragment with all parseable Artifact attributes."""
    allowed_fields = set(InternalApi().server_artifact_introspection())

    supports_ttl = "ttlIsInherited" in allowed_fields
    supports_tags = "tags" in allowed_fields

    ttl_duration_seconds = "ttlDurationSeconds" if supports_ttl else ""
    ttl_is_inherited = "ttlIsInherited" if supports_ttl else ""

    tags = "tags {name}" if supports_tags else ""

    # The goal is to move all artifact aliases fetches to the membership level in the future
    # but this is a quick fix to unblock the registry work
    aliases = (
        """aliases {
                artifactCollection {
                    project {
                        entityName
                        name
                    }
                    name
                }
                alias
            }"""
        if include_aliases
        else ""
    )

    return f"""
        fragment ArtifactFragment on Artifact {{
            id
            artifactSequence {{
                project {{
                    entityName
                    name
                }}
                name
            }}
            versionIndex
            artifactType {{
                name
            }}
            description
            metadata
            {ttl_duration_seconds}
            {ttl_is_inherited}
            {aliases}
            {tags}
            state
            currentManifest {{
                file {{
                    directUrl
                }}
            }}
            commitHash
            fileCount
            createdAt
            updatedAt
        }}
    """


def _gql_registry_fragment() -> str:
    return """
        fragment RegistryFragment on Project {
           id
            allowAllArtifactTypesInRegistry
            artifactTypes(includeAll: true) {
                edges {
                    node {
                        name
                    }
                }
            }
            name
            description
            createdAt
            updatedAt
        }
    """
